/*
DL2 Monitor: ./pa/paether2.c
Copyright (C) 2002, Rolf Jger, Ulm (Germany)

See the file README for information on usage and redistribution
of this source code file, and for a DISCLAIMER OF ALL WARRANTIES.
*/

/*----------------------------------------------------------------------------
 * Datalink Monitor (paether2): Protocol Analysis (DIX ethernet frame)
 *
 * Implementation Notes
 * P_* indicate frame positions of certain fields;
 * T_* indicate known ether types;
 * if an ether type is unkown, the analyzer returns -1, only;
 * for known ether types, the sub-analyzer is invoked.
 * default contains: ether-type 
 * 
 * Author: Rolf Jger,Ulm
 *----------------------------------------------------------------------------
 * YYYYMMDD  History
 * 20020427  ETHER2_Mchecksum: bug fixed. 
 * 20020330  created from scratch with pppoe, only. 
 * $Id: paether2.c,v 1.4 2002/04/29 13:01:03 dv Exp $
 *----------------------------------------------------------------------------
*/
# include <stdio.h>
# include <string.h>
# include "lib/dl2pa.h"
# include "lib/paether.h"
# include "pa/ether2.h"

// common to any DIX ethernet frame:
# define P_DSTADDR  0
# define P_SRCADDR  6
# define P_ETHERTYPE  0x0c
# define P_NEXT  0x0e

// known types: PPPoE
# define T_PPPoE_D  PAETH_T_PPPoE_D
# define T_PPPoE_S  PAETH_T_PPPoE_S

// default indications:
# define TI_PPPoE  "PPPoE"

//-exports--------------------------------------------------------------------
int dl2_paether2(Dl2_Frame *frame,int verbose) {
  Octet *buffer = frame->buffer;
  int length = frame->length, i, j;
  Doctet type;
  const char *typestring = PA_FT_UNKNOWN;
  int follow_frame = 0;

  // check for required packet length:
  if (length < P_NEXT+2) return -1;

  // check for ether type:
  type = str2doct(buffer+P_ETHERTYPE);
  
  switch (type) {
    case T_PPPoE_D:
    case T_PPPoE_S:
      frame->type = type;
      typestring = TI_PPPoE;
      follow_frame = (verbose & (ETHER2_Mfollow_pppoe|ETHER2_Mfollow_frame));
      break;
    default: return -1;
  }

  // destination address:
  if (verbose & ETHER2_Mdestination_addr) {
    printf("%s=",ETHER2_Fdestination_addr);
    for (i = 0; i < P_SRCADDR; i++) printf("%02x",buffer[i]);
  } else i = P_SRCADDR;

  // source address:
  if (verbose & ETHER2_Msource_addr) {
    printf(" %s=",ETHER2_Fsource_addr);
    for (; i < P_ETHERTYPE; i++) printf("%02x",buffer[i]);
  } else i = P_ETHERTYPE;

  // ether type:
  if (verbose & ETHER2_Mether_type) {
    printf(" %s=",ETHER2_Fether_type);
    for (; i < P_NEXT; i++) printf("%02x",buffer[i]);
  }
  else if (verbose & ETHER2_Mdefault) {
    printf(" %s",typestring);
  } i = P_NEXT;

  // payload length: tbd
  // payload data: tbd

  if (follow_frame) {
    switch (type) {
      case T_PPPoE_D:
      case T_PPPoE_S:
        frame->buffer = buffer+i;
        frame->length = length-i-2; 
        printf(" [");
        if ((j = dl2_papppoe(frame,pa_verbose(PA_PID_pppoe))) < 0) return -1;
        printf(" ]");
        i += j;
        break;
      default:
    }

    // checksum:
    if (verbose & ETHER2_Mchecksum) {
      printf(" %s=",ETHER2_Fchecksum);
      if (length > i+2) length = i+2;
      for (; i < length; i++) printf("%02x",buffer[i]);
    } 
  }
  return i;
} //dl2_paether2


//-locals---------------------------------------------------------------------

/***EOM***/
